# This file is designed to abstract out differences between versions of
# Microsoft Visual C++ and common compilation options for that
# environment.  It is designed to be independent of the package that it
# ships with.  Please don't attempt to introduce application-specific
# options here; keep these general.
#
# I will attempt to support versions of Microsoft Visual C++ right back
# to Optimizing Compiler version 8 (NMAKE 1.1) in this file.  That's
# pre-Visual C++ 2.0.

!IFDEF WIN16
!IF "$(WIN16)" == "1"
!IFNDEF WIN16_SUPPORT
!ERROR Attempting to compile with Win16, but package doesn't support it.
!ENDIF
WIN16=1
CPP=cl
link=link
lib=lib
!ELSE
!include <ntwin32.mak>
!ENDIF
!ELSE
!include <ntwin32.mak>
!ENDIF

#
# Supported options:
# MSC_VER =     n - Specify the version of Microsoft C++ - NOT Visual C++.
#               2.0 is 9, 4.0 is 10, 5.0 is 11, 6.0 is 12, 2002/2003 is 13,
#               Whidbey is 14, etc.  Default is (currently) 12.
!IFNDEF MSC_VER
MSC_VER = 12
!ENDIF

# MSC_ED =      std,pro,free - Specify the edition of Microsoft Visual C++;
#               use std for Standard Edition, pro for Professional Edition,
#               and free for Visual C++ Toolkit (free download.)  Default
#               is pro for MSC_VER <= 12, free for MSC_VER > 12.
!IFNDEF MSC_ED
!IF $(MSC_VER) > 12
MSC_ED=free
!ELSE
MSC_ED=pro
!ENDIF
!ENDIF

# CLIB_SHARED = 0 - Link statically against your C library.  Generally not
#               a good idea, but you *must* have this to compile with the
#               free Visual C++ Toolkit; and if you have newer compilers,
#               your customers might not have that version of the shared
#               C library installed, so using it is a bad idea.  Default
#               is 1 for MSC_VER <= 12; 0 afterwards.
!IFNDEF CLIB_SHARED
!IF $(MSC_VER) < 13
!IF $(MSC_VER) > 10
CLIB_SHARED = 1
!ELSE
CLIB_SHARED = 0
!ENDIF # MSC_VER > 10
!ELSE
CLIB_SHARED = 0
!ENDIF # MSC_VER < 13
!ENDIF # CLIB_SHARED

# DEBUG      = 1 - Enable debugging support.  Default is 0.
!IFNDEF DEBUG
DEBUG = 0
!ENDIF

# RELOC      = 1 - Enable relocation information in executables.  This
#              results in larger binaries, but they are Win32s (Windows
#              3.1) compatible.  Default is 0.
!IFNDEF RELOC
RELOC = 0
!ENDIF

# PC_HEADERS = 1 - Enable pre-compiled headers.  Depending on the
#              application, this may reduce build times, and will be handy
#              if you're developing that application.  The default is 0.
!IFNDEF PC_HEADERS
PC_HEADERS = 0
!ENDIF

# UNICODE    = 1 - Enable building a unicode application.  The default is 0.
#              This option may not be supported by all packages.
!IFNDEF UNICODE
UNICODE = 0
!ENDIF

!IF "$(UNICODE)" == "1"
!IFNDEF UNICODE_SUPPORT
!ERROR Attempting to compile with unicode, but package doesn't support it.
!ENDIF
!ENDIF

# WITH_3D    = 1 - Enable linking against ctl3d32.lib.  This is only used
#              on old versions of MSC to get 3D appearence; things already
#              look this way on newer versions.  The default is 0 except
#              on MSC 9 (Visual C++ 2.0).  This is the only version that
#              shipped with a default non-3D appearence.
!IFNDEF WITH_3D
!IF $(MSC_VER) == 9
WITH_3D = 1
!ELSE
WITH_3D = 0
!ENDIF # MSC_VER
!ENDIF # WITH_3D

# Sanity checking
!IF $(MSC_VER) < 8
!ERROR AutoWin supports Win16/Win32/Win64 compilers from version 8 or newer.
!ENDIF

!IFDEF MSC_MINIMUM_VER
!IF $(MSC_VER) < $(MSC_MINIMUM_VER)
!ERROR This package requires C version $(MSC_MINIMUM_VER) or greater.
!ENDIF
!ENDIF

BASE_LDFLAGS=
!IFDEF WIN16
BASE_LIBS=oldnames libw llibcew
BASE_CFLAGS=-W3 -nologo -D_WIN32_ -D_WIN16_ -AL
BASE_RCFLAGS=/d _WIN16_
!ELSE
BASE_LIBS=kernel32.lib
BASE_CFLAGS=-W3 -nologo -D_WIN32_
!ENDIF
BASE_ILFLAGS=

# Now we start generating everything.  Don't change lightly.

# Crazy variable-name cycling necessary for *very* old NMAKE's.
# These things didn't do direct substitution., ie CFLAGS=$(CFLAGS) is a
# circular reference.
!IFDEF WIN16
!IF $(DEBUG) == 2
CDEBUGFLAG=-Zi
LDEBUGFLAG=/CODEVIEW
DEBUG=2
!ELSE # DEBUG
!IF $(DEBUG) == 1
CDEBUGFLAG=-Ox -Gy -Zi
LDEBUGFLAG=/CODEVIEW
DEBUG=1
!ELSE # DEBUG
CDEBUGFLAG=-Ox -Gy
LDEBUGFLAG=
DEBUG=0
!ENDIF # DEBUG
!ENDIF # DEBUG

!IF "$(WITH_3D)" == "1"
3D_LIBS=ctl3d.lib
3D_CFLAGS=-DWITH_3D
3D_RCFLAGS=-d WITH_3D
!ELSE
3D_LIBS=
3D_CFLAGS=
3D_RCFLAGS=
!ENDIF

BSCMAKE=bscmake -nologo


!ELSE # WIN16
!IF "$(CLIB_SHARED)" == "1"
CLIBFLAG = -MD
!IF $(DEBUG) == 2
!IF $(MSC_VER) >= 10
!IF "$(MSC_ED)" != "free"
CLIBFLAG = -MDd
!ENDIF # MSC_ED
!ENDIF # MSC_VER
!ENDIF # DEBUG
CLIB_SHARED=1
!ELSE
CLIBFLAG = -MT
!IF $(DEBUG) == 2
!IF $(MSC_VER) >= 10
!IF "$(MSC_ED)" != "free"
CLIBFLAG = -MTd
!ENDIF # MSC_ED
!ENDIF # MSC_VER
!ENDIF # DEBUG
CLIB_SHARED=0
!ENDIF # CLIB_SHARED


!IF "$(DEBUG)" == "2"
CDEBUGFLAG=-Zi
DEBUG=2
!ELSE # DEBUG
!IF "$(DEBUG)" == "1"
!IF "$(MSC_ED)" == "std"
# Standard editions have no optimization.  Free ones do.  Go figure.
CDEBUGFLAG=-Gy -Zi
!ELSE # MSC_ED
CDEBUGFLAG=-Ox -Gy -Zi
!ENDIF # MSC_ED
!ELSE # DEBUG
!IF "$(MSC_ED)" == "std"
# Standard editions have no optimization.  Free ones do.  Go figure.
CDEBUGFLAG=-Gy
!ELSE # MSC_ED
CDEBUGFLAG=-Ox -Gy
!ENDIF # MSC_ED
!ENDIF # DEBUG
!ENDIF # DEBUG

LDEBUGFLAG=
!IF $(DEBUG) > 0
!IF $(MSC_VER) <= 10
LDEBUGFLAG=-DEBUG:full -DEBUGTYPE:both
!ELSE
LDEBUGFLAG=-DEBUG
!ENDIF # MSC_VER
!ENDIF # DEBUG

!IF $(MSC_VER) < 13
CVERFLAGS=-Gf
!ELSE
!IF $(MSC_VER) < 14
CVERFLAGS=-GF -Wp64
!ELSE
!IF "$(MSC_ED)" == "pro"
CVERFLAGS=-GF -Wp64 -D_CRT_SECURE_NO_DEPRECATE -D_CRT_NONSTDC_NO_DEPRECATE -GS
CVERLIBS=bufferoverflowu.lib
!ELSE
# Visual Studio 2005 express edition has NO 64-bit support, or security support.
CVERFLAGS=-GF -D_CRT_SECURE_NO_DEPRECATE -D_CRT_NONSTDC_NO_DEPRECATE
CVERLIBS=
!ENDIF # MSC_ED
!ENDIF # MSC_VER
!ENDIF # MSC_VER

!IF $(MSC_VER) == 8
# Picked up from compiler option on newer compilers
!IF "$(CLIB_SHARED)" == "1"
CVERLIBS=crtdll.lib
!ELSE 
CVERLIBS=libcmt.lib
!ENDIF #CLIB_SHARED
LVERFLAGS=-base:0x400000
!ELSE
LVERFLAGS=-nologo
ILVERFLAGS=/nologo
!ENDIF # MSC_VER

!IF "$(RELOC)" == "1"
!IF $(MSC_VER) > 10
LRELOCFLAGS=-fixed:no
!ELSE
LRELOCFLAGS=
!ENDIF
!ELSE
LRELOCFLAGS=-fixed
!ENDIF

!IF "$(WITH_3D)" == "1"
3D_LIBS=ctl3d32.lib
3D_CFLAGS=-DWITH_3D
3D_RCFLAGS=-d WITH_3D
!ELSE
3D_LIBS=
3D_CFLAGS=
3D_RCFLAGS=
!ENDIF

BSCMAKE=REM You don't have bscmake -- 
!IF $(MSC_VER) > 8
!IF "$(MSC_ED)" != "free"
BSCMAKE=bscmake -nologo
!ENDIF
!ENDIF
!ENDIF # Win16

!IF "$(PC_HEADERS)" == "1"
CPCHFLAGS=-YX
!ELSE
CPCHFLAGS=
!ENDIF

!IF "$(UNICODE)" == "1"
CUNIFLAGS=-DUNICODE -D_UNICODE
LD_CONSOLEAPP=-SUBSYSTEM:console -ENTRY:wmain
!ELSE
CUNIFLAGS=
LD_CONSOLEAPP=-SUBSYSTEM:console
!ENDIF

CFLAGS=$(BASE_CFLAGS) $(CLIBFLAG) $(CDEBUGFLAG) $(CVERFLAGS) $(CPCHFLAGS) $(3D_CFLAGS) $(CUNIFLAGS)
LDFLAGS=$(BASE_LDFLAGS) $(LDEBUGFLAG) $(LRELOCFLAGS) $(LVERFLAGS)
LIBS=$(BASE_LIBS) $(CVERLIBS) $(3D_LIBS)
ILFLAGS=$(BASE_ILFLAGS) $(ILVERFLAGS)
RCFLAGS=$(BASE_RCFLAGS) $(3D_RCFLAGS)

# The only issue here is to make sure DLL's are *never* fixed.
DLL_CFLAGS=$(CFLAGS)
DLL_LIBS=$(LIBS)
DLL_ILFLAGS=$(ILFLAGS)
DLL_RCFLAGS=$(RCFLAGS)
DLL_LDFLAGS=$(BASE_LDFLAGS) $(LDEBUGFLAG) $(LVERFLAGS)

BUILD=$(MAKE) -nologo -f Makefile.w32

w32bcfg.mak: w32bcfg-real AutoWin-custom display-config

display-config:
	@echo Current build settings:
	@type w32bcfg.mak

w32bcfg-real:
	@echo Regenerating Win32 Build configuration settings...
	@echo CFLAGS=$(CFLAGS) >w32bcfg.mak
	@echo LDFLAGS=$(LDFLAGS) >>w32bcfg.mak
	@echo ILFLAGS=$(ILFLAGS) >>w32bcfg.mak
	@echo RCFLAGS=$(RCFLAGS) >>w32bcfg.mak
	@echo LIBS=$(LIBS) >>w32bcfg.mak
	@echo LD_CONSOLEAPP=$(LD_CONSOLEAPP) >>w32bcfg.mak
	@echo DLL_CFLAGS=$(DLL_CFLAGS) >>w32bcfg.mak
	@echo DLL_LDFLAGS=$(DLL_LDFLAGS) >>w32bcfg.mak
	@echo DLL_ILFLAGS=$(DLL_ILFLAGS) >>w32bcfg.mak
	@echo DLL_RCFLAGS=$(DLL_RCFLAGS) >>w32bcfg.mak
	@echo DLL_LIBS=$(DLL_LIBS) >>w32bcfg.mak
	@echo BSCMAKE=$(BSCMAKE) >>w32bcfg.mak
	@echo MSC_VER=$(MSC_VER) >>w32bcfg.mak
	@echo MSC_ED=$(MSC_ED) >>w32bcfg.mak
	@echo CLIB_SHARED=$(CLIB_SHARED) >>w32bcfg.mak
	@echo DEBUG=$(DEBUG) >>w32bcfg.mak
	@echo PC_HEADERS=$(PC_HEADERS) >>w32bcfg.mak
	@echo UNICODE=$(UNICODE) >>w32bcfg.mak
	@echo WITH_3D=$(WITH_3D) >>w32bcfg.mak
	@echo BUILD=$(BUILD) >>w32bcfg.mak
!IFDEF WIN16
	@echo WIN16=$(WIN16) >>w32bcfg.mak
	@echo CPP=$(CPP) >>w32bcfg.mak
	@echo CC=$(CC) >>w32bcfg.mak
	@echo link=$(link) >>w32bcfg.mak
	@echo LIB=$(LIB)\\ >>w32bcfg.mak
!ENDIF

#
# It's over folks.  Below is just help.  You've missed all the guts of everything.
#

help: help-intro brief-clib-shared brief-debug brief-msc-ed brief-msc-ver brief-pc-headers brief-reloc brief-unicode brief-win16 brief-with-3d AutoWin-custom-help

help-intro:
	@echo "Summary of Windows configuration options."
	@echo.

brief-clib-shared:
	@echo "CLIB_SHARED=[0|1] - Defines whether to use a shared C library; default 1 for"
	@echo "Win32 MSC_VER 11 or 12 (use MSVCRT.DLL), 0 for everything else."
	@echo.

brief-debug:
	@echo "DEBUG=[0|1|2] - Defines whether to perform a debug build.  0 - No"
	@echo "debugging, 1 - hybrid (optimized with debug), 2 - full debug.  Default 0."
	@echo.

brief-msc-ed:
	@echo "MSC_ED=[free|std|pro] - Defines Visual C++ edition.  Default pro, except for"
  	@echo "MSC_VER 13 or newer, which is free."
	@echo.

brief-msc-ver:
	@echo "MSC_VER=[8|9|10|11|12|13|14] - Version of Microsoft C.  Use cl /? to"
	@echo "determine which version you have."
!IFDEF MSC_MINIMUM_VER
	@echo "This package requires $(MSC_MINIMUM_VER) or newer."
!ENDIF
	@echo.

brief-pc-headers:
	@echo "PC_HEADERS=[0|1] - Produce precompiled headers during build.  Default 0; only"
	@echo "normally gives performance benefit when apps are designed for it."
	@echo.

brief-reloc:
	@echo "RELOC=[0|1] - Produce relocatable executables.  DLLs are always relocatable."
	@echo "Default 0.  Ignored on Win16."
	@echo.

brief-unicode:
!IFDEF UNICODE_SUPPORT
	@echo "UNICODE=[0|1] - Produce a unicode executable.  Default is 0.  May not be"
	@echo "supported for all software."
!ENDIF

brief-win16:
!IFDEF WIN16_SUPPORT
	@echo "WIN16=[0|1] - Produce a Win16 build.  Default 0."
	@echo.
!ENDIF

brief-with-3d:
!IF $(MSC_VER) < 10
	@echo "WITH_3D=[0|1] - Link against ctl3d*.lib.  Default 0.  Only useful on Win16 or"
	@echo "old Win32 compilers (MSC_VER>=10 produce 3D widgets without this.)"
	@echo.
!ENDIF


