/*
 * Media.cpp
 *
 * This file contains all the routines for manipulating media, playing,
 * stopping etc.
 *
 * Copyright (c) Malcolm Smith 2001-2005.  No warranty is provided.
 */

#include "pch.h"

UINT wDeviceID;
DWORD dwIndex;

struct MCILink
{
	char szExt[16];
	char szDevice[32];
};

struct MCILink * lpLinks;
DWORD nLinks;
DWORD AllocatedLinks;


int ResetHandlers()
{
	lpLinks=(struct MCILink *)malloc(sizeof(struct MCILink)*10);
	nLinks=0;
	AllocatedLinks=10;
	return(0);
}

int AddHandler(LPSTR szExt,LPSTR szHandler)
{
	struct MCILink Temp;
	DWORD i;
	strcpy(Temp.szExt,szExt);
	strcpy(Temp.szDevice,szHandler);
	strupr(Temp.szExt);

	for (i=0;i<nLinks;i++) {
		if (!strcmp(Temp.szExt,lpLinks[i].szExt)) {
			memcpy(&lpLinks[i],&Temp,sizeof(struct MCILink));
			return(TRUE);
		}
	}
	nLinks++;
	if (nLinks>=AllocatedLinks) {
		AllocatedLinks+=10;
		lpLinks=(struct MCILink *)realloc(lpLinks,(size_t)AllocatedLinks*sizeof(struct MCILink));
	}
	memcpy(&lpLinks[nLinks-1],&Temp,sizeof(struct MCILink));

	return(0);
}

LPSTR GetHandler(LPSTR szExt)
{
	DWORD i;
	for (i=0;i<nLinks;i++) {
		if (!strcmp(szExt,lpLinks[i].szExt)) {
			return(lpLinks[i].szDevice);
		}
	}

	return(NULL);
}

LPSTR EnumHandlerExtensions(LPSTR szBuffer,LPSTR szSep)
{
	DWORD i;
	char szTemp[20];
	strcpy(szBuffer,"");
	for (i=0;i<nLinks;i++)
	{
		sprintf(szTemp,"%s*.%s",szSep,lpLinks[i].szExt);
		strlwr(szTemp);
		strcat(szBuffer,szTemp);
	}
	return(szBuffer);
}

int SetAudioPos(HWND hWndNotify)
{
	MCI_SEEK_PARMS mciSeekParms;
	MCI_PLAY_PARMS mciPlayParms;
    DWORD dwPos;

    dwPos=GetTrackerPos();

	mciSeekParms.dwTo=dwPos;
	mciSendCommand(wDeviceID,MCI_SEEK,MCI_WAIT|MCI_TO,(DWORD_PTR)&mciSeekParms);

	mciPlayParms.dwCallback = (DWORD_PTR) hWndNotify;
    mciPlayParms.dwFrom=dwPos;
	mciSendCommand(wDeviceID,MCI_PLAY,MCI_NOTIFY|MCI_FROM,(DWORD_PTR)&mciPlayParms);

	return(TRUE);
}

int PauseAudio()
{
	MCI_GENERIC_PARMS mciPauseParms;

	mciSendCommand(wDeviceID,MCI_PAUSE,MCI_WAIT,(DWORD_PTR)&mciPauseParms);
	return(TRUE);
}

int ResumeAudio()
{
	MCI_GENERIC_PARMS mciResumeParms;

	mciSendCommand(wDeviceID,MCI_RESUME,MCI_WAIT,(DWORD_PTR)&mciResumeParms);
	return(TRUE);
}




int UpdatePos()
{
	MCI_STATUS_PARMS mciStatusParms;
	mciStatusParms.dwItem=MCI_STATUS_POSITION;
	mciSendCommand(wDeviceID,MCI_STATUS,MCI_WAIT|MCI_STATUS_ITEM,(DWORD_PTR)&mciStatusParms);
	return(SetTrackerPos((DWORD)mciStatusParms.dwReturn));
}

DWORD playMP3File(HWND hWndNotify, LIST * List,int dwNewIndex)
{
	if (dwNewIndex>=0) {
		dwIndex=dwNewIndex;
	} else {
		switch (dwNewIndex) {
		case -1:
			dwNewIndex=(int)++dwIndex;
			SetActiveItem(dwNewIndex);
			break;
		case -2:
			if (!dwIndex) return(0);
			dwNewIndex=(int)--dwIndex;
			SetActiveItem(dwNewIndex);
			break;
		case -3:
			dwNewIndex=(int)dwIndex;
			break;
		}
	}
	if (dwNewIndex>=(int)List->dwItemsInList) return(0);
	//SetActiveItem(dwNewIndex);

	MCI_PLAY_PARMS mciPlayParms;
	MCI_SET_PARMS mciSetParms;
	MCI_STATUS_PARMS mciStatusParms;

	if (OpenStream(hWndNotify,GetListItem(List, dwNewIndex))) {
		mciSetParms.dwTimeFormat=MCI_FORMAT_MILLISECONDS;
		mciSendCommand(wDeviceID,MCI_SET,MCI_WAIT|MCI_SET_TIME_FORMAT,(DWORD_PTR)&mciSetParms);

		mciStatusParms.dwItem=MCI_STATUS_LENGTH;
		mciSendCommand(wDeviceID,MCI_STATUS,MCI_WAIT|MCI_STATUS_ITEM,(DWORD_PTR)&mciStatusParms);
		SetTrackerLength((DWORD)mciStatusParms.dwReturn);
		SetTrackerPos(0);

		if (IsTracker()&&IsWindowVisible(GethWndMain()))
			SetTimer(hWndNotify,TIMER_ID,TIMER_TICK,NULL);

		mciPlayParms.dwCallback = (DWORD_PTR) hWndNotify;
		if (!mciSendCommand(wDeviceID,MCI_PLAY,MCI_NOTIFY,(DWORD_PTR)&mciPlayParms))
		{
			return (TRUE);
		}

	}

    return (FALSE);
}

int CloseStream(HWND hWnd)
{
	if (wDeviceID) {
		mciSendCommand(wDeviceID, MCI_STOP, 0, 0); 
		mciSendCommand(wDeviceID, MCI_CLOSE, 0, 0);
		wDeviceID = 0;
		if (IsTracker())
			KillTimer(hWnd,TIMER_ID);
		return TRUE;
	} else
		return FALSE;
}

int OpenStream(HWND hWnd,LPSTR szFile)
{
	char szExt[16];
	LPSTR a;
	DWORD dwReturn;
	//MessageBox(hWnd,"OpenStream activated",product,64);
	if (wDeviceID) CloseStream(hWnd);
	MCI_OPEN_PARMS mciOpenParms;
    memset(&mciOpenParms, 0, sizeof(mciOpenParms));

	a=strrchr(szFile,'.');
	if (a) strcpy(szExt,a+1);
	strupr(szExt);

	mciOpenParms.lpstrElementName = szFile;
	mciOpenParms.lpstrDeviceType = GetHandler(szExt);
	if (!mciOpenParms.lpstrDeviceType) {
		if (dwReturn = mciSendCommand(0, MCI_OPEN,
		   MCI_OPEN_ELEMENT, (DWORD_PTR) &mciOpenParms))
		{
			MessageBox(hWnd,"Could not open output.  Check that an MCI handler is installed for this format.  Use the AddHandler command in your default.msc file to add support.",product,48);
			return(FALSE);
		}
	} else {
        char szTemp[200];
		if (dwReturn = mciSendCommand(MCI_DEVTYPE_OTHER, MCI_OPEN,
		   MCI_OPEN_TYPE | MCI_OPEN_ELEMENT, (DWORD_PTR)&mciOpenParms))
		{
			// Failed to open device. Don't close it; just return error.
            mciGetErrorString(dwReturn, szTemp, sizeof(szTemp));
            MessageBox(hWnd, szTemp, product, 16);
			return (FALSE);
		}
	}
	SetOpenFile(szFile);
	wDeviceID = mciOpenParms.wDeviceID;
	return(TRUE);

}

int IsPlaying()
{
	return(wDeviceID);
}

